#!/usr/bin/perl -w
package core::progress;

use strict;
use warnings;

use core::logger;
use File::Find;

use POSIX qw/floor/;

my $displayThreadExited:shared = 1;

# Description: Display the summary of perl migration
# Params: NA
# Return: NA
sub printsummary
{
    my $migrationoutput = shift;
    my $queriesfailed = 0;
    my $failedfiles = 0;
    my $migratedfiles = 0;
	
	if (defined($migrationoutput))
	{
		my @lines = split /\n/, $migrationoutput;
		foreach my $line (@lines) 
		{            
			if ($line =~ m/Number of queries failed :\s*(\d*)\s*/)
			{
				$line =~ m/Number of queries failed :\s*(\d*)\s*/;
				$queriesfailed = $1;
			}
			if ($line =~ m/Number of files failed :\s*(\d*)\s*/)
			{
				$line =~ m/Number of files failed :\s*(\d*)\s*/;
				$failedfiles = $1;
			}
		}
    }
	
    print "**************************************************************************\n";
	print "Number of Perl files in input directory : $core::teradatacore::allperlfilecount\n";
    print "Number of valid Perl files in input directory : $core::teradatacore::perlfilecount\n";    
    print "Number of Queries failed : $queriesfailed\n";    
    print "Number of SQL files failed : $failedfiles\n";            
    print "Number of Perl file failures while extracting sql contents : $core::teradatacore::extractionfailcount\n";
    print "Number of Perl file failures while merging sql contents : $core::teradatacore::mergefailcount\n";
    print "Number of Perl directory failures while extracting sql contents : $core::teradatacore::extractiondirfailcount\n";
    print "Number of Perl directory failures while merging sql contents : $core::teradatacore::mergedirfailcount\n";        
    
    if ($core::teradatacore::extractionfailcount == 0 && $core::teradatacore::mergefailcount == 0 && $core::teradatacore::extractiondirfailcount == 0 && $core::teradatacore::mergedirfailcount == 0)
    {
        print "Number of Perl files converted successfully : $core::teradatacore::perlfilecount\n";
    }        
    
    print "**************************************************************************\n";
}

# Description: Display progress of sql files migration completed in percentage
# Params:
# Return: NA
sub displaymigrationprogress
{
    my $totalfilecount = 0;        
    my $migratedfolder = $core::teradatacore::outputfolder.$core::teradatacore::directoryseparator."output";
    $displayThreadExited = 0;    
    
    # get the total sql file count
	find( sub {if ("$/"ne(/\.sql$/)){$totalfilecount++}},$core::teradatacore::location);    
    
    if ($totalfilecount == 0)
    {
        core::logger::writeinfoconsole("\rDSC progress : Files [0/0] Percentage [0\%]");   
        $displayThreadExited = 1;    
        core::logger::writeinfologmsg(__FILE__, __LINE__, "Exiting display thread");    
        return;
    }
        
    while(1)
    {
        my $currentmigratedfilecount = 0;
        
		# if directory is not created, wait till directory is created
		if (-e $migratedfolder)
		{
			# get the total sql file count
			find( sub {if ("$/"ne(/\.sql$/)){$currentmigratedfilecount++}},$migratedfolder);    
			
			if ($currentmigratedfilecount == $totalfilecount)
			{
				$currentmigratedfilecount = $currentmigratedfilecount - 1;
			}
			
			my $varpercentage = floor(($currentmigratedfilecount * 100) / $totalfilecount);
			core::logger::writeinfoconsole("\rDSC progress : Files [$currentmigratedfilecount/$totalfilecount] Percentage [$varpercentage\%]");        
			
			$| = 1 ;
		}
		
        sleep(1);        
        if($core::teradatacore::commandexecutioncompleted == 1)
        {
            last;
        }
    }
        
    $displayThreadExited = 1;    
    core::logger::writeinfologmsg(__FILE__, __LINE__, "Exiting display thread");    
    return;
}

# Description: Display progress of perl files migration completed in percentage
# Params:
# Return: NA
sub displayperlprogress
{
    my $stage = shift;
    $displayThreadExited = 0;    
    while(1)
    {
        if ($core::teradatacore::threadExit != $core::errorcodes::MIG_SUCCESS)
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Exiting display thread as some thread has failed with $core::teradatacore::threadExit");
            return;
        }
        
        my $varpercentage = floor(($core::teradatacore::migratedperlfilescount * 100) / $core::teradatacore::totalperlfilecount);
        if($stage eq "Stage1")
        {
            core::logger::writeinfoconsole("\rExtraction progress : Files [$core::teradatacore::migratedperlfilescount/$core::teradatacore::totalperlfilecount] Percentage [$varpercentage\%]");
        } 
        else
        {
            core::logger::writeinfoconsole("\rMerging progress : Files [$core::teradatacore::migratedperlfilescount/$core::teradatacore::totalperlfilecount] Percentage [$varpercentage\%]");
        }
        
        $| = 1 ;
        sleep(1);        
        if($core::teradatacore::totalperlfilecount == $core::teradatacore::migratedperlfilescount)
        {
            last;
        }
    }
    
    $displayThreadExited = 1;    
    core::logger::writeinfologmsg(__FILE__, __LINE__, "Exiting display thread $stage");    
    return;
}

# Description: clear current line, log and print message
# Params: inputmsg
# Return: NA 
sub clearcurrentlineprintlog
{
    my ($filename, $linenumber, $inputmsg) = @_;    
    core::logger::writeinfologmsg($filename, $linenumber, $inputmsg);
    if ($core::teradatacore::systemversion =~ /Microsoft/ || $core::teradatacore::systemversion =~ /MSW/)
    {
        require Win32::Console::ANSI;        
        print "\r\e[K$inputmsg\n";
    }
    else
    {
        print "\r\e[K$inputmsg\n";
    }
}

return 1;