import datetime
import logging
import os
import sys
from commands import getstatusoutput, getoutput

import gaussDBThread
import time

from run_utils import  rungaussDBAconnection, rungaussDBTconnection
from PropertyFile import readFile
from gaussDBThread import gaussDDLTask
from querycount import gettotalquerystatusbulk, writeandprintoutput, gettotalquerystatusblogic, \
    gettypecountbulk, gettypecountblogic, writeandprintoutputDBT
from shell_utils import setquerycount, getqueycount

configdata = dict()
outputfol = None
homepath = None
migrationtype = None
dbtype = None


def main():
    start = datetime.datetime.now()
    result = precheck()
    if result == 1:
        postcleanup(outputfol)
        return result
    # print '-------------Verification  Started---------------\n'
    print "Sql script execution on Gauss DB start time :", getoutput("date")

    importddltogauss(configdata)
    if dbtype == "gaussdba" :
        if "bulk" == migrationtype:
            gettotalquerystatusbulk(outputfol)
            gettypecountbulk(homepath)
        else:
            gettotalquerystatusblogic(outputfol)
            gettypecountblogic(homepath)
        writeandprintoutput(homepath)
    else:
        writeandprintoutputDBT(outputfol, homepath)

    relationexistancecheck()

    postcleanup(outputfol)

    print 'Gauss Execution Log file : {0} '.format(
        os.path.join(homepath, 'log','gaussexecutionlog.log'))
    print 'Gauss Execution Error Log file : {0} '.format(
        os.path.join(homepath, 'log','gaussexecutionerror.log'))
    finish = datetime.datetime.now()

    print 'Verification finished in {0} seconds'.format((finish - start).seconds)
    print '\n********************** Verification Completed ****************************\n\n'


def importddltogauss(configdata):
    """  Take the migrated script and execute on gaussdb """
    outputfolder = os.path.join(outputfol, 'output')
    totalquery = totalquerycountdir(outputfolder)
    if totalquery == 0:
        print 'No sql files found'
        logging.info("Total query is {0}".format(totalquery))
        return
    argument = ([totalquery])
    progressthreads = gaussDBThread.Thread(target=importprogress, args=argument)
    progressthreads.daemon = True
    progressthreads.start()
    logging.info("Gaussdb execution started")
    for dirs_file_name in sorted(os.walk(outputfolder)):
        currentcounttemp = getqueycount()
        totalfoldercounttemp = totalquerycountdirtemp(dirs_file_name)
        totalcounttemp = currentcounttemp + totalfoldercounttemp
        gaussDDLTask(dirs_file_name, configdata, homepath, outputfol)
        while True:
            if totalcounttemp == 0:
                break
            if getqueycount() >= totalcounttemp:
                break
            time.sleep(1)

    if progressthreads.isAlive():
        progressthreads.join()
    setquerycount(0)
    print "\rSql script execution on Gauss DB end time :", getoutput("date")

    logging.info("Gaussdb execution completed")


def importprogress(totalquery):
    """ Import to gaussdb progress"""

    time.sleep(1)
    if totalquery == 0:
        logging.error("Total query is {0}".format(totalquery))
        return

    while True:
        currentqueryno = getqueycount()

        num = "{0}%   [FILE ({1}/{2})]      ".format(((currentqueryno * 100) / totalquery), currentqueryno,
                                              totalquery)
        if currentqueryno >= totalquery:
            time.sleep(0.5)
            print "\r  100% Completed                            ",
            sys.stdout.flush()
            return
        print "\r Compilation Progress  {0}".format(num),
        sys.stdout.flush()
        time.sleep(1)


def relationexistancecheck():
    logpath = os.path.join(homepath, 'log', 'gaussexecutionerror.log')

    cmd = 'grep -w \"does not exist\" {0}|wc -l'.format(logpath)
    status, output = getstatusoutput(cmd)
    value = None
    if output.isdigit():
        value = int(output)
    if value is not None:
        if value == 0:
            cmd = "grep -w \"doesn't exist\" {0}|wc -l".format(logpath)
            status, output = getstatusoutput(cmd)
            if output.isdigit():
                value = int(output)

        if value > 0:
            print "[WARN]: We recognized that some dependent relations (Schema/Objects) are missing in the " \
                  "db/scripts, which is affecting the overall Success Rate. "
            print "Please check the errors in the below " \
                  "error log file and take necessary action.\n"



def totalquerycountdirtemp(dir_name):
    """ Returns the sql file count """
    return len(dir_name[2])


def totalquerycountdir(input_path):
    count = 0
    for dirs_file_name in os.walk(input_path):
        count = count + len(dirs_file_name[2])
    return count


def postcleanup(outputfol):
    folder = os.path.join(outputfol,'rangeMapper')
    if os.path.exists(folder):
        cmd = 'rm -rf {flename}*'.format(flename=folder)
        getstatusoutput(cmd)


def precheck():
    """ Precheck """
    try:
        argumentList = sys.argv
        # no of argumnet check
        if len(argumentList) != 5:
            print "Invalid number of arguments"
            return 1
        global migrationtype
        migrationtype = argumentList[1].lower().strip()

        # migration type  check
        if migrationtype != 'bulk' and migrationtype != 'blogic':
            print "Invalid migration type argument"
            return 1
        global homepath
        homepath = argumentList[2].strip()

        # home path check
        if not os.path.exists(homepath):
            print "Home path does not exist"
            return 1

        outputfolder = argumentList[3].strip()
        # output path check
        if not os.path.exists(outputfolder):
            print "Output folder path does not exist"
            return 1
        global outputfol
        outputfol = outputfolder

        path = os.path.join(homepath, 'config', 'gaussdb.properties')
        # config path check
        if not os.path.exists(path):
            print "Migration config path does not exist"
            return 1
        global configdata
        configdata = readFile(path)
        log_file_name = os.path.join(homepath, 'log', 'pythoncode.log')
        if not os.path.exists(log_file_name):
            cmd = "touch {0}".format(log_file_name)
            getstatusoutput(cmd)
        output_log_file_name = os.path.join(homepath, 'log', 'gaussexecutionlog.log')
        if os.path.exists(output_log_file_name):
            cmd = "rm {0} ".format(output_log_file_name)
            getstatusoutput(cmd)
        output_error_file_name = os.path.join(homepath, 'log', 'gaussexecutionerror.log')
        if os.path.exists(output_error_file_name):
            cmd = "rm {0} ".format(output_error_file_name)
            getstatusoutput(cmd)

       
        gaussdbpass = os.environ['gaussdb-password'].strip()
        os.environ['gaussdb-password'] = ''
        global dbtype
        dbtype = argumentList[4].strip()
        configdata['gaussdb-type'] = dbtype
        # Gauss DB Password check
        if '' == gaussdbpass:
            print 'GaussDB Password cannot be empty'
            return 1

        # update gaussdbpassword
        configdata['gaussdb-password'] = gaussdbpass

        logging.basicConfig(filename=log_file_name, level=logging.DEBUG,
                        format='%(asctime)s:%(levelname)s:%(filename)s:%(lineno)s:%(message)s')

        # DB connection check
        gaussresult = None
        if configdata['gaussdb-type'] == 'gaussdbt':
            gaussresult = rungaussDBTconnection(configdata)
        else:
            gaussresult = rungaussDBAconnection(configdata)
        if gaussresult[0] != 0:
            print 'Gauss DB connection failed'
            logging.error('DB connection failed :{0}'.format(gaussresult[1]))
            return 1
        return 0

    except Exception as error:
        logging.error(error)
        return 1

    


if __name__ == '__main__':
    try:
        result = main()
        if result == 1:
            logging.error('Gauss DB Execution Failed\n')
        else:
            logging.info('Task Completed Successfully')
    except (IOError, OSError) as error:
        #print str(error.strerror), '\n'
        logging.error('Exception ocuured {}'.format(error))
